#!/usr/bin/env python
#
############################################################################
#
# SCRIPT:    	FIDIMO Analysis for: Disentangling the effects of habitat suitability,
#		dispersal and fragmentation on the distribution of river fishes
#
# AUTHOR(S):    Johannes Radinger

# DATE:         2012-11-28
#
#############################################################################

import sys
import os
import atexit
import sqlite3
import csv

import grass.script as grass


#database-connection
env = grass.gisenv()
gisdbase = env['GISDBASE']
location = env['LOCATION_NAME']
mapset = env['MAPSET']
grass.run_command("db.connect",
		driver = "sqlite",
		database = os.path.join(gisdbase, location, mapset,'sqlite.db'))	
database = sqlite3.connect(os.path.join(gisdbase, location, mapset, 'sqlite.db'))
db = database.cursor()

# Getting resolution
res = int(grass.read_command("g.region",
				  flags = "m").strip().split('\n')[4].split('=')[1])  



# Base settings for fidimo
outflow_Treene="3505675,6026475"
river="Treene_raster@PERMANENT"
p = 0.67


species_dict = {'Phoxinus':(1.38,70), 'Pungtius':(1.17,65), 'Anguilla':(1,350), 'Rutiilus':(1.48,250), 'Gymnrnua':(0.84,120), 'Leucscus':(1.286,150), 'Tincinca':(1.45,200), 'Cobienia':(0.85,50), 'Gobiobio':(1.37,120), 'Percilis':(1.40,250), 'Salmario':(1.25,200), 'Gastatus':(1.69,51), 'Blicrkna':(2.49,200), 'Esoxcius':(1.39,400), 'Lampilis':(1,350), 'Lampneri':(0.56,160), 'Leucatus':(3.4,60)}

years=["1y","2y","3y","4y","5y","6y","7y","8y","9y"]


for i in species_dict:

	# Remove MASK
	grass.run_command("g.remove",
				rast="MASK")


	print(i)

	# Load fish samples
	grass.run_command("v.in.ascii",
			overwrite=True,
			input="/path/to/species/presence/points/files/"+i+"/CPUE100_"+i,
			output="CPUE100_"+i,
			fs=",",
			columns="SiteName VARCHAR, SiteID INT, CPUE100 DOUBLE, new_X DOUBLE, new_Y DOUBLE",
			x= 4,
			y= 5,
			skip=1)

	#transform species points raster to pres_abs map
	grass.run_command("v.to.rast",
			overwrite=True,
			input="CPUE100_"+i,
			output="CPUE100_"+i,
			use="attr",
			column="CPUE100")
				
	grass.mapcalc("$CPUE100_null=if($CPUE100>0,$CPUE100,null())",
		CPUE100_null = "CPUE100_null_"+i,
		CPUE100 = "CPUE100_"+i)


	for k in ["excl_barriers","incl_barriers"]:

		# Define if barriers are included or not
		if k == "incl_barriers":
			barriers="/path/to/barriers_file.csv"
		else:
			barriers=""

		for j in years:

			# Define Source populations (start with the pres_abs map)
			source_populations="CPUE100_null_"+i

		
			grass.message(_("The actual species is: "+i+". This run is "+k+" and "+str(j)+" starts now!"))
			
			grass.run_command("r.fidimo.py",
					flags="b",
					quiet = True,
					overwrite=True,
					river = river,
					coors = outflow_Treene,
					source_populations = source_populations,
					barriers = barriers,
					species= "Custom species",
					L = species_dict[i][1],
					AR = species_dict[i][0],
					p = p,
					statistical_interval = "no",
					truncation= "inf",
					seed=999,
					T = int(j[0])*365,
					output = "fidimo_"+i+"_"+j+"_"+k)

			grass.message("Run: "+str(j)+" for species: "+i+" is over")


